#!/bin/bash

BIN=/var/packages/Spreadsheet/target/tools/synofficehook
FILE_PKG_INFO=/var/packages/Spreadsheet/INFO
SHEET_FILE_NODE_SOCK=/run/synoffice/spreadsheet_node.sock
SHEET_PKG_DAEMON_NAME=pkg-synoffice-spreadsheet-node
DOC_FILE_NODE_SOCK=/run/synoffice/document_node.sock
DOC_PKG_DAEMON_NAME=pkg-synoffice-document-node
PRESENTATION_FILE_NODE_SOCK=/run/synoffice/presentation_node.sock
PRESENTATION_PKG_DAEMON_NAME=pkg-synoffice-presentation-node
PKG_MODVER="${sdkmodversion:-1.0}"

if [ -f "${FILE_PKG_INFO}" ]; then
	UTIL=/bin/get_key_value
	PKG_NAME=`${UTIL} ${FILE_PKG_INFO} package`
	PKG_VERSION=`${UTIL} ${FILE_PKG_INFO} version`
	PKG_VENDOR=`${UTIL} ${FILE_PKG_INFO} maintainer`
else
	echo "Office (Spreadsheet) package info file does not exist!";
	exit 1;
fi

GetPID() {
	if [ -f "/bin/systemctl" ]; then
		echo `systemctl show -p MainPID $1`
	else
		echo `initctl status $1 | egrep -o '([0-9]+)$' | head -n1`
	fi
}

SendRevokeAll() {
	if [ -S "${SHEET_FILE_NODE_SOCK}" ] ; then
		PID=$(GetPID ${SHEET_PKG_DAEMON_NAME})
		if [ ! -z "$PID" ] ; then
			kill -USR2 $PID
		fi
	fi
	if [ -S "${DOC_FILE_NODE_SOCK}" ] ; then
		PID=$(GetPID ${DOC_PKG_DAEMON_NAME})
		if [ ! -z "$PID" ] ; then
			kill -USR2 $PID
		fi
	fi
	if [ -S "${PRESENTATION_FILE_NODE_SOCK}" ] ; then
		PID=$(GetPID ${PRESENTATION_PKG_DAEMON_NAME})
		if [ ! -z "$PID" ] ; then
			kill -USR2 $PID
		fi
	fi
}

PostHandler() {
	if [ "$RESULT" = "0" ] ; then
		Post_Func
	fi
}

case "$1" in
	--sdk-mod-ver)
		echo ${PKG_MODVER};
	;;
	--name)
		echo ${PKG_NAME};
	;;
	--pkg-ver)
		echo ${PKG_VERSION};
	;;
	--vendor)
		echo ${PKG_VENDOR};
	;;
	--pre)
	;;
	--post)
		PostHandler
	;;
	*)
		echo "Usage: $0 --sdk-mod-ver|--name|--pkg-ver|--vendor|--pre|--post"
	;;
esac
