from openpyxl.styles.fills import FILL_NONE, FILL_SOLID
from openpyxl.styles.colors import COLOR_INDEX

COLOR_INDEX_LIST = list(COLOR_INDEX)
# 64: system foreground, mapping to black
COLOR_INDEX_LIST[64] = '00000000'
# 65: system background, mapping to white
COLOR_INDEX_LIST[65] = '00FFFFFF'


def get_color(color, wb):
    val = None
    color_type = color.type
    if color_type == 'rgb':
        val = color.rgb
    elif color_type == 'indexed':
        if color.indexed < len(COLOR_INDEX_LIST):
            val = COLOR_INDEX_LIST[color.indexed]
    elif color_type == 'theme' and wb.theme_colors:
        val = wb.theme_colors[color.theme]
        if color.tint != 0 and val:
            val = change_rgb_brightness(val, color.tint)
    return val[-6:].lower() if val else None


def change_rgb_brightness(rgb, tint):
    red = int(rgb[-6:-4], 16)
    green = int(rgb[-4:-2], 16)
    blue = int(rgb[-2:], 16)

    red += ((255 - red) if tint > 0 else red) * tint
    green += ((255 - green) if tint > 0 else green) * tint
    blue += ((255 - blue) if tint > 0 else blue) * tint
    red = min(max(int(red), 0), 255)
    green = min(max(int(green), 0), 255)
    blue = min(max(int(blue), 0), 255)
    return '{:x}{:x}{:x}'.format(red, green, blue)


def parse_font(f, wb):
    default_font_size = wb._fonts[0].sz
    font = {}
    if f.sz and f.sz != default_font_size and f.sz != 10:
        font['sz'] = int(f.sz)
    if f.b:
        font['b'] = True
    if f.i:
        font['i'] = True
    if f.strike:
        font['s'] = True
    if f.u:
        font['u'] = True
    if f.color:
        color = get_color(f.color, wb)
        if color and color != '000000':
            font['c'] = color
    return font if font else None


def parse_fill(f, wb):
    fill = {}
    bg_color = 'ffffff'
    if f.fill_type == FILL_SOLID and f.start_color:
        bg_color = get_color(f.start_color, wb)
    elif f.fill_type != FILL_NONE and f.end_color:
        bg_color = get_color(f.end_color, wb)
    if bg_color and bg_color != 'ffffff':
        fill['bg'] = bg_color
    return fill if fill else None


def parse_alignment(a):
    alignment = {}
    if a.horizontal == 'left':
        alignment['h'] = 'left'
    elif a.horizontal in ('justify', 'centerContinuous', 'center'):
        alignment['h'] = 'center'
    elif a.horizontal == 'right':
        alignment['h'] = 'right'
    if a.vertical == 'top':
        alignment['v'] = 'top'
    elif a.vertical in ('justify', 'center'):
        alignment['v'] = 'middle'
    elif a.vertical == 'bottom':
        pass
    if a.wrap_text is True:
        alignment['w'] = 'wrap'
    return alignment if alignment else None


def _get_border_side_color(side, wb):
    if side and side.style:
        color = None
        if side.color:
            color = get_color(side.color, wb)
        if color is None:
            color = '000000'
        return '#' + color if color else None
    return None


def parse_border(b, wb):
    border = [
        _get_border_side_color(b.top, wb),
        _get_border_side_color(b.right, wb),
        _get_border_side_color(b.bottom, wb),
        _get_border_side_color(b.left, wb)
    ]
    return {'b': border} if any(border) else None
