<?php
/**
 * This file is part of PHPPresentation - A pure PHP library for reading and writing
 * presentations documents.
 *
 * PHPPresentation is free software distributed under the terms of the GNU Lesser
 * General Public License version 3 as published by the Free Software Foundation.
 *
 * For the full copyright and license information, please read the LICENSE
 * file that was distributed with this source code. For the full list of
 * contributors, visit https://github.com/PHPOffice/PHPPresentation/contributors.
 *
 * @link        https://github.com/PHPOffice/PHPPresentation
 * @copyright   2009-2015 PHPPresentation contributors
 * @license     http://www.gnu.org/licenses/lgpl.txt LGPL version 3
 */

namespace PhpOffice\PhpPresentation\Shape;

use PhpOffice\PhpPresentation\AbstractShape;
use PhpOffice\PhpPresentation\ComparableInterface;
use PhpOffice\PhpPresentation\Style\Border;

/**
 * Line shape
 */
class Line extends AbstractShape implements ComparableInterface
{
    private $tailEnd = false;
    private $headEnd = false;

    /**
     * Create a new \PhpOffice\PhpPresentation\Shape\Line instance
     */
    public function __construct()
    {
        parent::__construct();
        $this->getBorder()->setLineStyle(Border::LINE_SINGLE);
    }

    /**
     * Get hash code
     *
     * @return string Hash code
     */
    public function getHashCode()
    {
        return md5($this->getBorder()->getLineStyle() . parent::getHashCode() . __CLASS__);
    }

    public function getTailEnd()
    {
        return $this->tailEnd;
    }

    public function setTailEnd()
    {
        $this->tailEnd = true;
    }

    public function getHeadEnd()
    {
        return $this->headEnd;
    }

    public function setHeadEnd()
    {
        $this->headEnd = true;
    }
}
