<?php
namespace PhpOffice\PhpWord\Writer\HTML\Element;

/**
 * ListItemRun element HTML writer
 *
 * @since 0.10.0
 */
class ListItemRun extends TextRun
{
    /**
     * Previous list items' strcuture
     * ex:
     *     <ul>
     *         <li>a</li>
     *         <ol>
     *             <li>a-1</li> <== current ListItemRun
     *         </ol>
     *         <li>b</li>
     *     </ul>
     * this variable sould be ('ul', 'ol')
     */
    private $parent_tags = array();

    /**
     * Write list item run
     *
     * @return string
     */
    public function write()
    {
        $content = '';

        if ($this->prev_element instanceof \PhpOffice\PhpWord\Element\ListItemRun) {
            $this->parent_tags = $this->prev_writer->getParentTags();
        } else {
            $this->parent_tags = array();
        }

        $this->write_recursive($content, count($this->parent_tags));

        return $content;
    }

    private function write_recursive(&$content, $curr_level)
    {
        $target_level = $this->element->getDepth() + 1;
        $tag = $this->getListItemTag();

        if ($target_level > $curr_level) {
            $content .= "<" . $tag . ">";
            array_push($this->parent_tags, $tag);
            $this->write_recursive($content, $curr_level + 1);
        } else if ($target_level < $curr_level) {
            $content .= "</" . array_pop($this->parent_tags) . ">";
            $this->write_recursive($content, $curr_level - 1);
        } else if (end($this->parent_tags) === $tag) {
            $writer = new Container($this->parentWriter, $this->element);

            $content .= "<li>";
            $content .= $writer->write();
            $content .= "</li>";

            if (!($this->next_element instanceof \PhpOffice\PhpWord\Element\ListItemRun)) {
                while (count($this->parent_tags) > 0) {
                    $content .= "</" . array_pop($this->parent_tags) . ">";
                }
            }
        } else {
            while (count($this->parent_tags) > 0) {
                $content .= "</" . array_pop($this->parent_tags) . ">";
            }
            $this->write_recursive($content, 0);
        }
    }

    private function getListItemTag()
    {
        if ($this->element->getStyle()->getListType() === \PhpOffice\PhpWord\Style\ListItem::TYPE_NUMBER) {
            return 'ol';
        } else if (\PhpOffice\PhpWord\Style::getStyle('PHPWordList' . ($this->element->getStyle()->getNumId() - 1))->getLevels()[0]->getFormat() === 'decimal') {
            return 'ol';
        } else {
            return 'ul';
        }
    }

    public function getParentTags()
    {
        return $this->parent_tags;
    }
}
