<?php
namespace PhpOffice\PhpWord\Writer\Word2007\Element;

use PhpOffice\PhpWord\Element\Graph as GraphElement;

class Graph extends Inline
{
    public function write()
    {
        $xmlWriter = $this->getXmlWriter();
        $element = $this->getElement();
        if (!$element instanceof GraphElement) {
            return;
        }
        $id = $element->getId();
        $config = $element->getConfig();
        $width = intval($config['width'] * 9525);
        $height = intval($config['height'] * 9525);

        $this->writeCommentRangeStart($xmlWriter, $element);
        $this->writeRevisionOpenTag($xmlWriter, $element);

        $xmlWriter->startElement('w:r');
        $xmlWriter->startElement('w:drawing');
        switch ($config['alignment']) {
            case 'break':
                $xmlWriter->startElement('wp:anchor');
                $xmlWriter->writeAttribute('distT', 0);
                $xmlWriter->writeAttribute('distB', 0);
                $xmlWriter->writeAttribute('distL', 114300);
                $xmlWriter->writeAttribute('distR', 114300);
                $xmlWriter->writeAttribute('simplePos', 0);
                $xmlWriter->writeAttribute('relativeHeight', intval($config['order']) + 251658240);
                $xmlWriter->writeAttribute('behindDoc', 0);
                $xmlWriter->writeAttribute('locked', 0);
                $xmlWriter->writeAttribute('layoutInCell', 1);
                $xmlWriter->writeAttribute('allowOverlap', 1);
                $xmlWriter->writeElementBlock('wp:simplePos', array('x' => 0, 'y' => 0));
                $xmlWriter->startElement('wp:positionH');
                $xmlWriter->writeAttribute('relativeFrom', 'page');
                $xmlWriter->startElement('wp:posOffset');
                $xmlWriter->text(intval($config['left'] * 9525));
                $xmlWriter->endElement(); // wp:posOffset
                $xmlWriter->endElement(); // wp:positionH
                $xmlWriter->startElement('wp:positionV');
                $xmlWriter->writeAttribute('relativeFrom', 'page');
                $xmlWriter->startElement('wp:posOffset');
                $xmlWriter->text(intval($config['top'] * 9525));
                $xmlWriter->endElement(); // wp:posOffset
                $xmlWriter->endElement(); // wp:positionV
                // This must be put after <wp:extent>
                // What the fuck Microsoft Word
                // $xmlWriter->writeElementBlock('wp:wrapTopAndBottom', array());
                break;
            case 'inline':
            default:
                $xmlWriter->startElement('wp:inline');
                break;
        }
        $xmlWriter->writeElementBlock('wp:extent', array('cx' => $width, 'cy' => $height));
        switch ($config['alignment']) {
            case 'break':
                $xmlWriter->writeElementBlock('wp:wrapTopAndBottom', array());
                break;
            case 'inline':
            default:
                break;
        }
        $xmlWriter->writeElementBlock('wp:docPr', array('id' => $id, 'name' => "{$config['prst']} {$id}"));
        $xmlWriter->startElement('a:graphic');
        $xmlWriter->writeAttribute('xmlns:a', 'http://schemas.openxmlformats.org/drawingml/2006/main');
        $xmlWriter->startElement('a:graphicData');
        $xmlWriter->writeAttribute('uri', 'http://schemas.microsoft.com/office/word/2010/wordprocessingShape');
        $xmlWriter->startElement('wps:wsp');
        $xmlWriter->writeAttribute('xmlns:wps', 'http://schemas.microsoft.com/office/word/2010/wordprocessingShape');
        $xmlWriter->writeElementBlock('wps:cNvSpPr', array());
        $xmlWriter->startElement('wps:spPr');
        $xmlWriter->startElement('a:xfrm');
        if ($config['flip_h']) {
            $xmlWriter->writeAttribute('flipH', 1);
        }
        if ($config['flip_v']) {
            $xmlWriter->writeAttribute('flipV', 1);
        }
        if ($config['deg'] !== 0) { // TODO take care about floating point
            $xmlWriter->writeAttribute('rot', intval($config['deg'] * 60000));
        }
        $xmlWriter->writeElementBlock('a:off', array('x' => 0, 'y' => 0)); // TODO
        $xmlWriter->writeElementBlock('a:ext', array('cx' => $width, 'cy' => $height));
        $xmlWriter->endElement(); // a:xfrm
        $xmlWriter->startElement('a:prstGeom'); // TODO custGeom
        $xmlWriter->writeAttribute('prst', $config['prst']);
        $xmlWriter->startElement('a:avLst');
        foreach ($config['customized_points'] as $k => $v) {
            $xmlWriter->writeElementBlock('a:gd', array('name' => $k, 'fmla' => "val " . intval($v)));
        }
        $xmlWriter->endElement(); // a:avLst
        $xmlWriter->endElement(); // a:prstGeom
        $xmlWriter->startElement('a:solidFill');
        $xmlWriter->writeElementBlock('a:srgbClr', array('val' => trim($config['fill'], '#')));
        $xmlWriter->endElement(); // a:solidFill
        $xmlWriter->startElement('a:ln');
        $xmlWriter->writeAttribute('align', 'ctr');
        $xmlWriter->writeAttribute('cap', 'flat');
        $xmlWriter->writeAttribute('cmpd', 'sng');
        $xmlWriter->writeAttribute('w', intval($config['stroke-width'] * 9525));
        $xmlWriter->startElement('a:solidFill');
        $xmlWriter->writeElementBlock('a:srgbClr', array('val' => trim($config['stroke'], '#')));
        $xmlWriter->endElement(); // a:solidFill
        $xmlWriter->writeElementBlock('a:prstDash', array('val' => 'solid'));
        $this->writeLineEnd($xmlWriter, 'a:headEnd', $config['start_marker']);
        $this->writeLineEnd($xmlWriter, 'a:tailEnd', $config['end_marker']);
        $xmlWriter->endElement(); // a:ln
        $xmlWriter->endElement(); // wps:spPr
        $xmlWriter->startElement('wps:bodyPr');
        $xmlWriter->writeAttribute('anchor', 'ctr');
        $xmlWriter->writeAttribute('anchorCtr', 0);
        $xmlWriter->endElement(); // wps:bodyPr
        $xmlWriter->endElement(); // wps:wsp
        $xmlWriter->endElement(); // a:graphicData
        $xmlWriter->endElement(); // a:graphic
        $xmlWriter->endElement(); // wp:inline or wp:anchor
        $xmlWriter->endElement(); // w:drawing
        $xmlWriter->endElement(); // w:r

        $this->writeRevisionCloseTag($xmlWriter, $element);
        $this->writeCommentRangeEnd($xmlWriter, $element);
    }

    private function writeLineEnd($xmlWriter, $tag, $type) {
        switch ($type) {
            case 'arrow':
                $xmlWriter->writeElementBlock($tag, array('type' => 'triangle'));
                break;
            // case 'none':
            default:
                break;
        }
    }
}
